-------------------------------------------------------------------------------
-- Steamwheedle Helper By Lockslap (US, Bleeding Hollow)
--------------------------------------------------------------------------------
SteamHelper = LibStub("AceAddon-3.0"):NewAddon("Steamwheedle Helper", "AceEvent-3.0", "AceConsole-3.0")

-- libraries we'll use
local L = LibStub("AceLocale-3.0"):GetLocale("Steamwheedle Helper", false)
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")

-- globals
SteamHelper.name = L["SteamwheedleHelper"]
SteamHelper.title = L["Steamwheedle Helper"]
SteamHelper.version = GetAddOnMetadata("SteamwheedleHelper", "Version")

-- vars we'll use later
local waypoints = {}
local enabledString = ("|cff00ff00%s|r"):format(L["Enabled"])
local disabledString = ("|cffff0000%s|r"):format(L["Disabled"])
local zoneID = 950	-- zone id for nagrand
local factionID = 1711	-- faction id for steamwheedle preservation society
local repStandings = {
	[1] = L["Hated"],
	[2] = L["Hostile"],
	[3] = L["Unfriendly"],
	[4] = L["Neutral"],
	[5] = L["Friendly"],
	[6] = L["Honored"],
	[7] = L["Revered"],
	[8] = L["Exalted"],
}

-- options table
local db, defaults = {}, {
	char = {
		enabled = true,
		notify = false,
		show = {
			minimap = false,
			worldmap = false,
			arrow = true,
			nextWay = 15,
		},
	},
}
local fields = {"Title", "Version", "Author", "X-Category", "X-License", "X-Email", "Email", "eMail", "X-Website", "X-Credits", "X-Localizations", "X-Donate", "X-Paypal"}
local fieldName = "|cff33ff99%s:|r"
local options = {
	type = "group",
	args = {
		general = {
			type = "group",
			name = SteamHelper.name,
			get = function(info) return db[info[#info]] end,
			set = function(info, value)
				db[info[#info]] = value
				SteamHelper:RemoveWaypoints()
				if db.enabled then
					SteamHelper:LoadWaypoints()
				end
			end,
			args = {
				header = {
					type = "header",
					order = 1,
					name = (L["Version:  |cff99ff33%s|r"]):format(SteamHelper.version),
					width = "full",
				},
				description = {
					type = "description",
					order = 2,
					name = L["A simple addon to manage the known locations for Highmaul Reliquaries for Steamwheedle Preservation Society reputation."],
					width = "full",
				},
				enabled = {
					type = "toggle",
					order = 3,
					name = L["Enabled"],
				},
				notify = {
					type = "toggle",
					order = 4,
					name = L["Display Notifications"],
					disabled = function() return not db.enabled end,
				},
				reload = {
					type = "toggle",
					order = 5,
					name = L["Automatically Reload"],
					desc = L["Automatically reload the waypoints when you have completed them all."],
					disabled = function() return not db.enabled end,
				},
				show = {
					type = "group",
					order = 99,
					guiInline = true,
					name = L["Show Waypoints On..."],
					disabled = function() return not db.enabled end,
					get = function(info) return db.show[info[#info]] end,
					set = function(info, value)
						db.show[info[#info]] = value
						SteamHelper:RemoveWaypoints()
						if db.enabled then
							SteamHelper:LoadWaypoints()
						end
					end,
					args = {
						arrow = {
							type = "toggle",
							order = 1,
							name = L["Crazy Arrow"],
							desc = L["Show waypoint on TomTom's crazy arrow."],
						},
						minimap = {
							type = "toggle",
							order = 2,
							name = L["Minimap"],
							desc = L["Show waypoints on the minimap."],
						},
						worldmap = {
							type = "toggle",
							order = 3,
							name = L["World Map"],
							desc = L["Show waypoints on the world map."],
						},
						nextWay = {
							type = "range",
							order = 4,
							name = L["Arrival Distance"],
							desc = L["Distance, in yards, from the current waypoints before the next one is loaded.  Default is 15 yards."],
							min = 1, max = 100, step = 1,
						},
					},
				},
			},
		},
		about = {
			type = "group",
			name = L["About"],
			args = {},
		},
	},
}

local function RepStatus(id)
	local name, _, standing, barMin, barMax, barValue, _, _, _, _, _, _, _ = GetFactionInfoByID(id)
	barValue = barValue - barMin
	barMax = barMax - barMin
	return name, barValue, barMax, standing, math.floor(barValue / barMax * 100)
end

function SteamHelper:Print(str)
	DEFAULT_CHAT_FRAME:AddMessage(("|cff33ff99SWH|r: %s"):format(str))
end

function SteamHelper:InArray(value)
	if #waypoints == 0 then return end
	for i = 1, #waypoints do
		if waypoints[i] == value then
			return i
		end
	end
	return false
end

function SteamHelper:LoadWaypoints()
	local zone = GetCurrentMapAreaID()
	local zoneName = GetMapNameByID(zone)
	
	if db.enabled and zone == zoneID then
		for _, point in pairs(self.coords) do
			local opts = {
				title = (L["Steamwheedle Helper\n%s"]):format(zoneName),
				persistent = false,
				minimap = db.show.minimap,
				world = db.show.worldmap,
				crazy = db.show.arrow,
				cleardistance = 0,
				arrivaldistance = 0,
				callbacks = {},
			}
			opts.callbacks = TomTom:DefaultCallbacks(opts)
			opts.callbacks.distance[db.show.nextWay] = function(event, uid, range, distance, lastDistance)
				if lastDistance > distance then
					local inArray = SteamHelper:InArray(uid)
					TomTom:RemoveWaypoint(uid)
					if inArray ~= false then
						waypoints[inArray] = nil
					end
					
					-- notify about waypoints completion
					if db.notify and #waypoints == 0 then
						if db.reload then
							SteamHelper:Print((L["Completed |cff33ff99%d|r waypoints.  Automatically reloading."]):format(#SteamHelper.coords))
							SteamHelper:RemoveWaypoints()
							SteamHelper:LoadWaypoints()
						else
							SteamHelper:Print((L["Completed |cff33ff99%d|r waypoints.  Type |cff33ff99/steam reload|r to reload them."]):format(#SteamHelper.coords))
						end
					end
				end
			end
			waypoints[#waypoints + 1] = TomTom:AddMFWaypoint(zone, 0, point.x / 100, point.y / 100, opts)
			
			if db.notify then
				self:Print((L["Loaded |cff33ff99%d|r waypoints."]):format(#SteamHelper.coords))
			end
		end
	else
		self:RemoveWaypoints()
	end
end

function SteamHelper:RemoveWaypoints()
	if #waypoints == 0 then return end
	for i = 1, #waypoints do
		if waypoints[i] then
			TomTom:RemoveWaypoint(waypoints[i])
		end
	end
end

function SteamHelper:ZONE_CHANGED_NEW_AREA(event, ...)
	self:LoadWaypoints()
end
SteamHelper:RegisterEvent("ZONE_CHANGED_NEW_AREA")

function SteamHelper:ChatCommand(args)
	local command, value = self:GetArgs(args, 2)
	
	if command == "reload" or command == "rl" then
		self:Print((L["Reloaded |cff33ff99%d|r waypoints."]):format(#SteamHelper.coords))
		self:RemoveWaypoints()
		self:LoadWaypoints()
	
	elseif command == "config" then
		InterfaceOptionsFrame_OpenToCategory(self.frames.general)
	
	elseif command == "rep" or command == "reputation" then
		local repString = "%s |cff33ff99%d/%d %s (%d%%)|r %s |cff33ff99%s|r."
		local name, value, max, standing, percent = RepStatus(factionID)
		self:Print(repString:format(L["You are"], value, max, repStandings[standing], percent, L["with the"], name))
		
	elseif command == "help" or command == "?" then
		self:Print(L["Usage:  |cff33ff99/steam|r |cffffff00<argument>|r or |cff33ff99/swh|r |cffffff00<argument>|r"])
		self:Print(L["Available Commands:  toggle, help, ?, config, rep, reload"])
		
	elseif command == "toggle" or not command then
		db.enabled = not db.enabled
		self:Print((L["Waypoints have been %s"]):format(db.enabled == true and enabledString or disabledString))
		self:RemoveWaypoints()
		if db.enabled then
			self:LoadWaypoints()
		end
		
	else
		self:Print(L["Usage:  |cff33ff99/steam|r |cffffff00<argument>|r or |cff33ff99/swh|r |cffffff00<argument>|r"])
		self:Print(L["Available Commands:  toggle, help, ?, config, rep, reload"])
	end
end

function SteamHelper:OnEnable()
	self:RegisterChatCommand("steam", "ChatCommand")
	self:RegisterChatCommand("swh", "ChatCommand")
	
	if db.enabled then
		self:LoadWaypoints()
	end
end

function SteamHelper:OnDisable()
	self:UnregisterChatCommand("steam")
	self:UnregisterChatCommand("swh")
	self:RemoveWaypoints()
end

function SteamHelper:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SteamwheedleHelperDB", defaults)
	db = self.db.char
	
	-- setup tomtom
	TomTom.profile.arrow.enable = true
	TomTom.profile.arrow.arrival = 25
	TomTom.profile.arrow.setclosest = true
	
	-- setup the options frame
	local i = 1
	for _, field in pairs(fields) do
		local value = GetAddOnMetadata(self.name, field)
		if value then
			options.args.about.args[field] = {
				type = "description",
				order = i,
				width = "normal",
				name = fieldName:format(field:gsub("X%-", "")),
			}
			i = i + 1
			options.args.about.args[("%s_value"):format(field)] = {
				type = "description",
				order = i,
				width = "double",
				name = value,
			}
			i = i + 1
		end
	end
	
	AceConfigRegistry:RegisterOptionsTable(self.title, options)
	self.frames = {}
	self.frames.general = AceConfigDialog:AddToBlizOptions(self.title, nil, nil, "general")
	self.frames.about = AceConfigDialog:AddToBlizOptions(self.title, L["About"], self.title, "about")
end